#ifndef __CUBTILE__
#define __CUBTILE__

#include "face.H"
#include "util.H"

#include "glTransfo.H"

/**
 * Les donnes geometriques
 */
typedef struct _ConfigGeo{
	int nl;
	int nc;
	cv::Rect rectH;
	cv::Rect rectB;
	cv::Rect rectG;
	cv::Rect rectD;
	cv::Rect rectM;

	_ConfigGeo(){}

	// Voir dimensions.png
	_ConfigGeo(int x1, int x2, int x3, int x4, int x5, int x6,
	          int y1, int y2, int y3, int y4, int y5, int y6,
	          int _nl,
	          int _nc);	

} ConfigGeo;


/**
 * @class Cubtile
 * @brief La classe de plus haut niveau pour la gestion du cubtile. Contient la boucle principale...
 */
class Cubtile {

private:
	// Tableau de 5 faces avec les conventions suivantes
	// faceHaut   =		myFace[0]
	// faceBas    =		myFace[1]
	// faceMilieu =		myFace[2]
	// faceDroit  =		myFace[3]
	// faceGauche =		myFace[4]

	/**
	 * Les 5 faces identifiées
	 */
	FaceH fH;
	FaceB fB;
	FaceC fC;
	FaceD fD;
	FaceG fG;

	// Classe pour la gestion de la transformation via OpenGL
	glTransfo gltransfo;


	// le tableau des 5 faces
	Face* faces[5];

	// La source video
	SourceVideo2 video;

	// une frame tampon qui permet de récuperer l'image
	Matrx frame;

	// Données de positionnement géométrique de l'image
	ConfigGeo confGeo;


	// une frame transformée
	Matrx frame_transfo;

	//Pour mettre ou pas les coordonnées entre 0 et 1
	bool echelle;
	


protected:
	/**
	 * Configure la ROI, le contraste, confGeo, confCam, etc...
	 */
	void configurerCubtile();

	/**
	 * @brief : demarre la caméra
	 */
	void lancerCamera();

	/**
	 * @brief : Arrete la caméra
	 */
	void arreterCamera();

	/**
	 * @brief : charge une première fausse image pour pouvoir : nécessaire pour mainloop;
	 */
	void amorcer();

	/**
	 * @brief : rend à la caméra la dernière frame amorcée proprement : necessaire à la sortie de mainloop
	 */
	void desamorcer();

	/**
	 * @brief selectionne la prochaine frame du flux vidéo 
	 *
	 */
	int nextFrame();

	/**
	 * @brief applique la transformation à l'image
	 *
	 */
	void transformation();


	/**
	 * @brief extrait les doigts
	 * @param algo l'algorithme à utiliser
	 */
	void extraction(CHOIX_ALGO algo);

	/**
	 * @brief Meme chose avec des mesures
	 */
	void extractionMesure(CHOIX_ALGO algo, int r);

	/**
	 * @brief Identifie les doigts sur les frames successives
	 */
	void tracker();

	/**
	 * @brief Meme chose avec des mesures
	 */
	void trackerMesure();

	/**
	 * @brief signale les nouveaux événements
	 */
	void signaler();

public:

	/**
	 * @brief Constructeur par défaut
	 *
	 * Sous entend que la caméra est branchée
	 */ 
	Cubtile();

	/**
	 * @brief Constructeur à partir des paramètres de la lignes de commande
	 */ 
	Cubtile(int argc, char * argv[], bool ech = true);


	/**
	 * @brief calibrer le cubtile
	 */ 
	void calibrer();
		

	/**
	 * @brief la boucle principale
	 * @param arret la condition de sortie de la boucle
	 * @param id la face à afficher (aucune par défaut)
	 * @param etape : quelle étape du traitement à afficher
	 */
	void mainLoop(volatile bool  & arret, ID_FACE id = FACEH, AFFICHAGE_FACE etape = NONE, CHOIX_ALGO algo = DISQUE);


	/**
	 * @brief Meme chose avec des mesures
	 */
	void mainLoopMesure(volatile bool & arret, ID_FACE id, AFFICHAGE_FACE etape, CHOIX_ALGO algo, int r = 0);

	/**
	 * @brief : Ajouter une action à l'évenement Motion
	 */
	void addEventMotionAll(void (*ptr_fctDisappear)(InfoEvent));
		
	/**
	 * @brief : Ajouter une action à l'évenement Appear
	 */
	void addEventAppearAll(void (*ptr_fctAppear)(InfoEvent));

	/**
	 * @brief : Ajouter une action à l'évenement Disappear
	 */
	void addEventDisappearAll(void (*ptr_fctDisappear)(InfoEvent));

	void addEventMotion   (ID_FACE id, void (*ptr_fctMotion)(InfoEvent));
	void addEventAppear   (ID_FACE id, void (*ptr_fctAppear)(InfoEvent));
	void addEventDisappear(ID_FACE id, void (*ptr_fctDisappear)(InfoEvent));
	

	/**
	 * @brief : destructeur qui désalloue la mémoire et quitte proprement en éteigant la caméra
	 */
	~Cubtile();


};




#endif
