#ifndef __FACE_H
#define __FACE_H

/**
 * @file face.H
 *
 * Fichier de la classe Face
 */

#include <cv.h>

#include "matx.H"
#include "tracker.H"
#include "event.H"


#include "util.H"


#include "extraction.H"


typedef struct _ConfFace{
	cv::Rect rectangle;
}ConfFace;

/**
 * @class Face
 * @brief Correspond à une face du cubtile
 */
class Face{
protected:

	// quelle face ?
	ID_FACE idf;

	// Le type d'affichage de la face
	AFFICHAGE_FACE affichage;

	//Le temps d'affichage de chaque face
	unsigned int temps_affichage;

	// Différents buffer :

	Matrx image; // L'image de base, rognée
	Matrx imageRedressee; // L'image dans le bon sens
	Matrx imageTransformee; // L'image carrée transformée

	/* Mapping de la face */
	int tailleMap; // taille du mapping (discrétisation)
    int h; // distance entre 2 noeuds de la grille en ndp
	coordPix ** Map; // mapping de l'image déformée (trapèze)

	/* Points de contrôles du trapèze déformé */
	coordPix A;
	coordPix B;
	coordPix C;
	coordPix D;
	coordPix E;
	coordPix F;

	Matrx imageRecadree; // La base du trapeze coincide avec la largeur de l'img
	Matrx imageDroite; // Elimination de l'effet perspective

	Extraction extract;

	Tracker track; // le suivis des points

	std::vector<cv::Point2d> detection; // Les points utilisés pour la détection


public:

	Face();


	/* Alloue la mémoire pour l'image Transformée et le maillage */
	void setMap(int nbDiscretisation, int pasNbPix);

	/**
	 * Mutateur permettant de rentrer les coordonnées des 6 points
	 * nécessaires à la transformation 
	 */
	void setCoord(double xA, double yA,
				  double xB, double yB,
				  double xC, double yC,
				  double xD, double yD,
				  double xE, double yE,
				  double xF, double yF);

	/**
	 * @brief Effectue les différentes allocations necessaires
	 * et configure le type de données
	 */
	void configurer();

	/**
	 * @brief Permet d'avoir un affichage,
	 * @param affa l'etape à afficher
	 * @param temps_aff le temps où la frame reste affichée. 0 pour que attendre un evenement clavier pour passer a la suivante
	 */ 
	void configurerAffichage(AFFICHAGE_FACE aff = NONE, unsigned int temps_aff = 1);

	/**
	 * Met l'image dans la bonne position (pour les faces de cotés uniquement);
	 */ 
	virtual void redresser();

	/**
	 * Génère automatiquement un maillage de l'image déformée
	 * et le stocke dans l'attribut Map
	 */
	void mapping(coordPix A, coordPix B, coordPix C,
				 coordPix D, coordPix E, coordPix F);

	/**
	 * Transforme l'image déformée en une image carrée
	 */
	void transforme();

	/**
	 * Elimine l'effet de perspective sur l'image recadrée
     * @param xe abscisse du coin supérieur gauche du trapèze
     * @param ye ordonnée du coin supérieur gauche du trapèze
	 * @param xf abscisse du coin supérieur droit du trapèze
	 * @param yf ordonnée du coin supérieur droit du trapèze
     */
	void supprPersp(int xe, int ye, int xf, int yf) ;


	// Attention au type !!
	/**
	 * ajoute une nouvelle frame à la face
	 */
	void initNextFrame(Matrx nextFrame);

	/**
	 * @brief execute l'extraction des points des images redressées
	 */
	void extraction(CHOIX_ALGO algo);

	void extractionMesure(CHOIX_ALGO algo, int r);
	/**
	 * @brief Effectue l'identification
	 */
	void tracker(bool echelle);

	/**
	 * @brief Effectue l'identification
	 */
	void signaler();

	/**
	 * @brief ajoute un évenement motion
	 */
	void addEventMotion(Event E);

	/**
	 * @brief ajoute un evenement Appear
	 */
	void addEventAppear(Event E);

	/**
	 * @brief ajoute un evenement Disappear
	 */
	void addEventDisappear(Event E);

	Matrx& getImage();

};


/**
 * @class FaceH
 * @brief Correspond à la face Haute du cubtile
 */

class FaceH: public Face{
public:

	FaceH();
	
	void configurerGeo();
	void redresser();
	void configurer();

};


/**
 * @class FaceB
 * @brief Correspond à la face basse du cubtile
 */

class FaceB: public Face{

public:

	FaceB();
	
	void configurerGeo();
	void redresser();
	void configurer();

};


/**
 * @class FaceG
 * @brief Correspond à la face Gauche du cubtile
 */

class FaceG: public Face{

public:

	FaceG();
	
	void configurerGeo();
	void redresser();
	void configurer();

};


/**
 * @class FaceD
 * @brief Correspond à la face droite du cubtile
 */

class FaceD: public Face{

public:

	FaceD();
	
	void configurerGeo();
	void redresser();
	void configurer();

};

/**
 * @class FaceC
 * @brief Correspond à la face centre du cubtile
 */

class FaceC: public Face{

public:

	FaceC();
	
	void configurerGeo();
	void redresser(); // Ne doit rien faire
	void configurer();
	void transforme();
};

#endif
